/*
 * blanco Framework Copyright (C) 2005 Tosiki Iga
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 */
package blanco.soap.dotnet.wsdl2cs;

import blanco.cg.BlancoCgObjectFactory;
import blanco.cg.valueobject.BlancoCgMethod;
import blanco.cg.valueobject.BlancoCgParameter;
import blanco.commons.util.BlancoNameAdjuster;
import blanco.commons.util.BlancoStringUtil;
import blanco.ig.expander.ClassExpander;
import blanco.ig.expander.IgType;
import blanco.ig.expander.IgValue;
import blanco.ig.expander.field.FieldExpander;
import blanco.ig.expander.implementor.IgStatement;
import blanco.ig.expander.method.MethodExpander;
import blanco.soap.dotnet.wsdl2cs.resourcebundle.BlancoSoapDotNetWsdl2CsResourceBundle;
import blanco.wsdl.valueobject.BlancoWsdlWebService;
import blanco.wsdl.valueobject.BlancoWsdlWebServiceOperation;

public class BlancoSoapDotnetWsdl2CsExpandTelegramProcessorProxy extends
        ClassExpander {
    private BlancoWsdlWebService fWsdl = null;

    private final BlancoSoapDotNetWsdl2CsResourceBundle bundle = new BlancoSoapDotNetWsdl2CsResourceBundle();

    public BlancoSoapDotnetWsdl2CsExpandTelegramProcessorProxy(
            final IgType type, final BlancoWsdlWebService wsdl) {
        super(type);
        this.fWsdl = wsdl;
    }

    protected void expandClassStruct() {
        final String superClassName = fWsdl.getPackage() + "."
                + fWsdl.getWebServiceId();
        setSuperClass(new IgType(superClassName));

        final String strNamespace = fWsdl.getNamespace();

        // .NET Framework 2.0p̃Ame[VB
        final String serviceBindingName = fWsdl.getWebServiceId().substring(0,
                fWsdl.getWebServiceId().length() - "Service".length())
                + "Binding";
        getCgClass()
                .getAnnotationList()
                .add(
                        "System.CodeDom.Compiler.GeneratedCodeAttribute(\"wsdl\", \"2.0.50727.42\")");
        getCgClass().getAnnotationList().add(
                "System.Web.Services.WebServiceAttribute(Namespace=\""
                        + strNamespace + "\")");
        getCgClass().getAnnotationList().add(
                "System.Web.Services.WebServiceBindingAttribute(Name=\""
                        + serviceBindingName + "\", Namespace=\""
                        + strNamespace + "\")");

        // L̋Lڂ .NET Framework 1.1p̃Ame[VΉłB
        // getCgClass().getAnnotationList().add(
        // "WebService(Namespace=\"" + fWsdl.getNamespace() + "\")");

        addFileComment("̃NX blancoɂ莩Ă܂B");
        getJavaDoc().addLine("̃NXWebT[rXdւ̃vLVNXłB");
        getJavaDoc().addLine(
                ".NET Framework SDK  wsdl.exe WebT[rX["
                        + fWsdl.getWebServiceId() + "]̒ۃNX[" + superClassName
                        + "]pĂ܂B");

        addImport(new IgType("System.Exception"));
        addImport(new IgType("System.Web.Services.WebService"));

        if (BlancoStringUtil.null2Blank(bundle.getWsProxyUsing01()).trim()
                .length() > 0) {
            addImport(new IgType(bundle.getWsProxyUsing01()));
        }
        if (BlancoStringUtil.null2Blank(bundle.getWsProxyUsing02()).trim()
                .length() > 0) {
            addImport(new IgType(bundle.getWsProxyUsing02()));
        }
        if (BlancoStringUtil.null2Blank(bundle.getWsProxyUsing03()).trim()
                .length() > 0) {
            addImport(new IgType(bundle.getWsProxyUsing03()));
        }
        if (BlancoStringUtil.null2Blank(bundle.getWsProxyUsing04()).trim()
                .length() > 0) {
            addImport(new IgType(bundle.getWsProxyUsing04()));
        }
        if (BlancoStringUtil.null2Blank(bundle.getWsProxyUsing05()).trim()
                .length() > 0) {
            addImport(new IgType(bundle.getWsProxyUsing05()));
        }

        final FieldExpander fieldLog = new FieldExpander(new IgType(
                "log4net.ILog"), "fLog") {
            protected boolean isStatic() {
                return false;
            }
        };
        fieldLog.setDefaultStatement(new IgStatement(
                "LogManager.GetLogger(\"Server\")"));
        fieldLog.getJavaDoc().addLine("log4net̃IuWFNg̃CX^XB");
        fieldLog.getJavaDoc()
                .addLine("ƗێړI邽߂ɁAlog4netAPI𒼐ڌĂяoĂ܂B");
        addField(fieldLog);

        if (BlancoStringUtil.null2Blank(bundle.getWsProxyConstructor01())
                .trim().length() > 0) {
            final BlancoCgObjectFactory cgFactory = BlancoCgObjectFactory
                    .getInstance();
            final BlancoCgMethod cgMethod = cgFactory.createMethod(getCgClass()
                    .getName(), "vLVNX̃RXgN^B");
            cgMethod.setConstructor(true);
            getCgClass().getMethodList().add(cgMethod);

            cgMethod.getLineList().add(
                    cgFactory.createLine(bundle.getWsProxyConstructor01()));

            if (BlancoStringUtil.null2Blank(bundle.getWsProxyConstructor02())
                    .trim().length() > 0) {
                cgMethod.getLineList().add(
                        cgFactory.createLine(bundle.getWsProxyConstructor02()));
            }
            if (BlancoStringUtil.null2Blank(bundle.getWsProxyConstructor03())
                    .trim().length() > 0) {
                cgMethod.getLineList().add(
                        cgFactory.createLine(bundle.getWsProxyConstructor03()));
            }
            if (BlancoStringUtil.null2Blank(bundle.getWsProxyConstructor04())
                    .trim().length() > 0) {
                cgMethod.getLineList().add(
                        cgFactory.createLine(bundle.getWsProxyConstructor04()));
            }
            if (BlancoStringUtil.null2Blank(bundle.getWsProxyConstructor05())
                    .trim().length() > 0) {
                cgMethod.getLineList().add(
                        cgFactory.createLine(bundle.getWsProxyConstructor05()));
            }
        }

        for (int indexOperation = 0; indexOperation < fWsdl.getOperationList()
                .size(); indexOperation++) {
            final BlancoWsdlWebServiceOperation operation = (BlancoWsdlWebServiceOperation) fWsdl
                    .getOperationList().get(indexOperation);

            addMethod(new MethodExpander(operation.getName()) {
                public void setupSignature() {
                    // .NET Framework 2.0p̃Ame[VB
                    getCgMethod().getAnnotationList().add(
                            "System.Web.Services.WebMethodAttribute()");
                    getCgMethod()
                            .getAnnotationList()
                            .add(
                                    "System.Web.Services.Protocols.SoapDocumentMethodAttribute(\""
                                            + strNamespace
                                            + "/"
                                            + operation.getName()
                                            + "\", Use=System.Web.Services.Description.SoapBindingUse.Literal, ParameterStyle=System.Web.Services.Protocols.SoapParameterStyle.Bare)");
                    getCgMethod().getAnnotationList()
                            .add(
                                    "return: System.Xml.Serialization.XmlElementAttribute(\""
                                            + operation.getOutput().getName()
                                            + "\", Namespace=\"" + strNamespace
                                            + "\")");

                    // .NET Framework 1.1p̃Ame[VB
                    // getCgMethod().getAnnotationList().add("WebMethod");

                    getCgMethod().setOverride(true);

                    setReturnType(new IgType(operation.getOutput()
                            .getPackageOfJava(), operation.getOutput()
                            .getName()));
                    addArgument(new IgValue(
                            new IgType(operation.getInput().getPackageOfJava(),
                                    operation.getInput().getName()), "input"));

                    // p[^ɃAme[VǉB
                    final BlancoCgParameter parameter = (BlancoCgParameter) getCgMethod()
                            .getParameterList().get(0);
                    parameter.getAnnotationList()
                            .add(
                                    "System.Xml.Serialization.XmlElementAttribute(\""
                                            + operation.getInput().getName()
                                            + "\", Namespace=\"" + strNamespace
                                            + "\")");

                    getJavaDoc().addLine("̃\bhWebT[rX̃\bhłB");
                }

                public void implement() {
                    getData()
                            .addLine(
                                    operation.getOutput().getName()
                                            + " output = null;");

                    getData().addLine("// f[^x[Xڑ擾܂B");
                    getData().addLine(
                            "System.Data.SqlClient.SqlConnection conn = null;");
                    getData()
                            .addLine(
                                    "System.Data.SqlClient.SqlTransaction tran = null;");
                    getData().addLine("try ");
                    getData().addLine("{");
                    getData().addLine("// f[^x[XڑJn܂B");
                    if (BlancoStringUtil.null2Blank(
                            bundle.getWsProxyDbConnect01()).length() > 0) {
                        getData().addLine(bundle.getWsProxyDbConnect01());
                    }
                    if (BlancoStringUtil.null2Blank(
                            bundle.getWsProxyDbConnect02()).length() > 0) {
                        getData().addLine(bundle.getWsProxyDbConnect02());
                    }
                    if (BlancoStringUtil.null2Blank(
                            bundle.getWsProxyDbConnect03()).length() > 0) {
                        getData().addLine(bundle.getWsProxyDbConnect03());
                    }
                    if (BlancoStringUtil.null2Blank(
                            bundle.getWsProxyDbConnect04()).length() > 0) {
                        getData().addLine(bundle.getWsProxyDbConnect04());
                    }
                    if (BlancoStringUtil.null2Blank(
                            bundle.getWsProxyDbConnect05()).length() > 0) {
                        getData().addLine(bundle.getWsProxyDbConnect05());
                    }
                    getData().addLine("tran = conn.BeginTransaction();");
                    getData().addLine("");
                    getData().addLine("// ۂ̏Ăяo܂B");
                    getData()
                            .addLine(
                                    "Abstract"
                                            + BlancoNameAdjuster
                                                    .toClassName(operation
                                                            .getName())
                                            + "TelegramProcessor telegramProcessor = new "
                                            + BlancoNameAdjuster
                                                    .toClassName(operation
                                                            .getName())
                                            + "TelegramProcessor();");
                    getData().addLine("telegramProcessor.SetConnection(conn);");
                    getData()
                            .addLine("telegramProcessor.SetTransaction(tran);");
                    getData().addLine(
                            "output = telegramProcessor.Execute(input);");
                    getData().addLine("");
                    getData().addLine(
                            "// dƂă[obNtOĂȂꍇɂ̂݃R~bgȂ܂B");
                    getData().addLine(
                            "if (telegramProcessor.GetRollback() == false)");
                    getData().addLine("{");
                    getData().addLine("tran.Commit();");
                    getData().addLine("// R~bg𔭍ŝŁAnullZbgď܂B");
                    getData().addLine("tran = null;");
                    getData().addLine("}");
                    getData().addLine("");
                    getData().addLine("return output;");
                    getData().addLine("}");

                    // System.Data.SqlClient.SqlException Lb`
                    getData().addLine(
                            "catch(System.Data.SqlClient.SqlException ex)");
                    getData().addLine("{");
                    getData().addLine("//Console.WriteLine(ex.ToString());");
                    getData().addLine("// MOs܂B");
                    getData().addLine(
                            "fLog.Fatal(\"SQLO܂B\" + ex.ToString());");
                    getData().addLine("// OIuWFNǵÂ܂܃X[܂B");
                    getData().addLine("throw ex;");
                    getData().addLine("}");

                    // SoapException Lb`
                    addUsingType(new IgType(
                            "System.Web.Services.Protocols.SoapException"));
                    getData().addLine("catch(SoapException ex)");
                    getData().addLine("{");
                    getData().addLine("//Console.WriteLine(ex.ToString());");
                    getData().addLine("// MOs܂B");
                    getData()
                            .addLine(
                                    "fLog.Fatal(\"WebT[rXO܂B\" + ex.ToString());");
                    getData().addLine("// OIuWFNǵÂ܂܃X[܂B");
                    getData().addLine("throw ex;");
                    getData().addLine("}");

                    // System.SystemException Lb`
                    getData().addLine("catch(System.SystemException ex)");
                    getData().addLine("{");
                    getData().addLine("//Console.WriteLine(ex.ToString());");
                    getData().addLine("// MOs܂B");
                    getData()
                            .addLine(
                                    "fLog.Fatal(\"System.SystemException܂B\" + ex.ToString());");
                    getData().addLine("// OIuWFNǵÂ܂܃X[܂B");
                    getData().addLine("throw ex;");
                    getData().addLine("}");

                    // System.Exception Lb`
                    getData().addLine("catch(System.Exception ex)");
                    getData().addLine("{");
                    getData().addLine("//Console.WriteLine(ex.ToString());");
                    getData().addLine("// MOs܂B");
                    getData()
                            .addLine(
                                    "fLog.Fatal(\"System.Exception܂B\" + ex.ToString());");
                    getData().addLine("// OIuWFNǵÂ܂܃X[܂B");
                    getData().addLine("throw ex;");
                    getData().addLine("}");

                    getData().addLine("finally");
                    getData().addLine("{");
                    getData().addLine("try");
                    getData().addLine("{");
                    getData().addLine("if(tran != null)");
                    getData().addLine("{");
                    getData().addLine(
                            "// gUNVIuWFNgAR~bg̏ꍇɂ̂݃[obN܂B");
                    getData()
                            .addLine(
                                    "// ADO.NETŃR~bgɃ[obNƗO邽 ̂悤ɂĂ܂B");
                    getData().addLine("tran.Rollback();");
                    getData().addLine("}");
                    getData().addLine("}");
                    getData().addLine("finally");
                    getData().addLine("{");
                    getData().addLine("if(conn != null)");
                    getData().addLine("{");
                    getData().addLine("// f[^x[XڑI܂B");
                    if (BlancoStringUtil.null2Blank(
                            bundle.getWsProxyDbClose01()).length() > 0) {
                        getData().addLine(bundle.getWsProxyDbClose01());
                    }
                    if (BlancoStringUtil.null2Blank(
                            bundle.getWsProxyDbClose02()).length() > 0) {
                        getData().addLine(bundle.getWsProxyDbClose02());
                    }
                    if (BlancoStringUtil.null2Blank(
                            bundle.getWsProxyDbClose03()).length() > 0) {
                        getData().addLine(bundle.getWsProxyDbClose03());
                    }
                    if (BlancoStringUtil.null2Blank(
                            bundle.getWsProxyDbClose04()).length() > 0) {
                        getData().addLine(bundle.getWsProxyDbClose04());
                    }
                    if (BlancoStringUtil.null2Blank(
                            bundle.getWsProxyDbClose05()).length() > 0) {
                        getData().addLine(bundle.getWsProxyDbClose05());
                    }
                    getData().addLine("}");
                    getData().addLine("}");
                    getData().addLine("}");
                }
            });
        }
    }
}
